﻿#region Copyright ©2011-2013, SIVECO Romania SA - All Rights Reserved
// ======================================================================
// Copyright ©2011-2013 SIVECO Romania SA - All Rights Reserved
// ======================================================================
// This file and its contents are protected by Romanian and International
// copyright laws. Unauthorized reproduction and/or distribution of all
// or any portion of the code contained herein is strictly prohibited
// and will result in severe civil and criminal penalties.
// Any violations of this copyright will be prosecuted
// to the fullest extent possible under law.
// ======================================================================
// THIS COPYRIGHT NOTICE MAY NOT BE REMOVED FROM THIS FILE.
// ======================================================================
#endregion

#region References
using System;
using System.IO;
using System.Runtime.InteropServices;
using System.Text;
using Cnas.Siui.CidGen;
#endregion

namespace Cnas.Siui.Cnp2Cid
{
    public static class CidGen
    {
        [DllImport( "CidGen32.dll", CharSet = CharSet.Ansi, CallingConvention = CallingConvention.Cdecl )]
        [return: MarshalAs( UnmanagedType.Bool )]
        private static extern bool GetCid( [MarshalAs( UnmanagedType.LPStr )] string pid,
                                           [MarshalAs( UnmanagedType.LPStr )] StringBuilder cid );

        private delegate string CidGenDelegate( string x );
        private static readonly CidGenDelegate CidGenFunc = null;

        static CidGen()
        {
            try
            {
                if( File.Exists( "CidGen32.dll" ) )
                {
                    if( string.IsNullOrEmpty( CidGen.GetCidHashWin( string.Empty ) ) == false )
                    {
                        CidGenFunc = CidGen.GetCidHashWin;
                    }
                }
            }
            finally
            {
                if( CidGenFunc == null && IsNet45OrNewer() == false )
                {
                    CidGenFunc = CidGen.GetCidHashNet;
                }
            }
        }

        public static string GetCidHash( string pid )
        {
            return CidGenFunc != null ? CidGenFunc( pid ) : null;
        }

        internal static string GetCidHashNet( string pid )
        {
            return CryptoHash.GetCidHash( pid );
        }


        internal static string GetCidHashWin( string pid )
        {
            var cid = new StringBuilder();
            return GetCid( pid, cid ) ? cid.ToString() : null;
        }

        private static bool IsNet45OrNewer()
        {
            // Class "ReflectionContext" exists from .NET 4.5 onwards.
            return Type.GetType( "System.Reflection.ReflectionContext", false ) != null;
        }
    }
}